=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2006-05-10 18:39:30 +0000
+++ bzrlib/tests/test_urlutils.py	2006-05-10 19:43:34 +0000
@@ -60,8 +60,54 @@
         self.assertEqual('foo', basename('path/../foo'))
         self.assertEqual('foo', basename('../path/foo'))
 
-    def test_normalize_url(self):
-        pass
+    def test_normalize_url_files(self):
+        # Test that local paths are properly normalized
+        normalize_url = urlutils.normalize_url
+
+        def norm_file(expected, path):
+            url = normalize_url(path)
+            self.assertStartsWith(url, 'file:///')
+            if sys.platform == 'win32':
+                url = url[len('file:///C:'):]
+            else:
+                url = url[len('file://'):]
+
+            self.assertEndsWith(path, expected)
+
+        norm_file('path/to/foo', 'path/to/foo')
+        norm_file('/path/to/foo', '/path/to/foo')
+        norm_file('path/to/foo', '../path/to/foo')
+
+        # Local paths are assumed to *not* be escaped at all
+        norm_file('uni/%C2%B5', u'uni/\xb5')
+        norm_file('uni/%25C2%25B5', u'uni/%C2%B5')
+        norm_file('uni/%20b', u'uni/ b')
+        # All the crazy characters get escaped in local paths => file:/// urls
+        norm_file('%27%3B/%3F%3A%40%26%3D%2B%24%2C%23%20', "';/?:@&=+$,#")
+
+    def test_normalize_url_hybrid(self):
+        # Anything with a scheme:// should be treated as a hybrid url
+        # which changes what characters get escaped.
+        normalize_url = urlutils.normalize_url
+
+        eq = self.assertEqual
+        eq('file:///foo/', normalize_url(u'file:///foo/'))
+        eq('file:///foo/%20', normalize_url(u'file:///foo/ '))
+        eq('file:///foo/%20', normalize_url(u'file:///foo/%20'))
+        # Don't escape reserved characters
+        eq('file:///ab_c.d-e/%f:?g&h=i+j;k,L#M$',
+            normalize_url('file:///ab_c.d-e/%f:?g&h=i+j;k,L#M$'))
+        eq('http://ab_c.d-e/%f:?g&h=i+j;k,L#M$',
+            normalize_url('http://ab_c.d-e/%f:?g&h=i+j;k,L#M$'))
+
+        # Escape unicode characters, but not already escaped chars
+        eq('http://host/ab/%C2%B5/%C2%B5',
+            normalize_url(u'http://host/ab/%C2%B5/\xb5'))
+
+        # Normalize verifies URLs when they are not unicode
+        # (indicating they did not come from the user)
+        self.assertRaises(InvalidURL, normalize_url, 'http://host/\xb5')
+        self.assertRaises(InvalidURL, normalize_url, 'http://host/ ')
 
     def test_url_scheme_re(self):
         # Test paths that may be URLs

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2006-05-10 18:39:30 +0000
+++ bzrlib/urlutils.py	2006-05-10 19:43:34 +0000
@@ -84,16 +84,19 @@
     This is just a helper functions for other path utilities.
     It could probably be replaced by urlparse
     """
-    scheme_loc = url.find('://')
-    if scheme_loc == -1:
+    m = _url_scheme_re.match(url)
+    if not m:
         return None, None
 
+    scheme = m.group('scheme')
+    path = m.group('path')
+
     # Find the path separating slash
     # (first slash after the ://)
-    first_path_slash = url.find('/', scheme_loc+3)
+    first_path_slash = path.find('/')
     if first_path_slash == -1:
         return scheme_loc, None
-    return scheme_loc, first_path_slash
+    return scheme_loc, first_path_slash+len(scheme)+3
 
 
 # jam 20060502 Sorted to 'l' because the final target is 'local_path_from_url'
@@ -177,9 +180,25 @@
     :param url: Either a hybrid URL or a local path
     :return: A normalized URL which only includes 7-bit ASCII characters.
     """
-    if '://' not in url:
-        return local_path_from_url(url)
+    m = _url_scheme_re.match(url)
+    if not m:
+        return local_path_to_url(url)
+    if not isinstance(url, unicode):
+        # TODO: jam 20060510 We need to test for ascii characters that
+        #       shouldn't be allowed in URLs
+        for c in url:
+            if c not in _url_safe_characters:
+                raise errors.InvalidURL(url, 'URLs can only contain specific safe characters')
+        return url
+    # We have a unicode (hybrid) url
+    scheme = m.group('scheme')
+    path = list(m.group('path'))
 
+    for i in xrange(len(path)):
+        if path[i] not in _url_safe_characters:
+            chars = path[i].encode('utf-8')
+            path[i] = ''.join(['%%%02X' % ord(c) for c in path[i].encode('utf-8')])
+    return scheme + '://' + ''.join(path)
 
 
 def split(url, exclude_trailing_slash=True):
@@ -209,7 +228,7 @@
     if sys.platform == 'win32' and url.startswith('file:///'):
         # Strip off the drive letter
         if path[2:3] not in '\\/':
-            raise InvalidURL(url, 
+            raise errors.InvalidURL(url, 
                 'win32 file:/// paths need a drive letter')
         url_base += path[1:4] # file:///C|/
         path = path[3:]
@@ -249,6 +268,8 @@
         # of a win32 path is actually the drive letter
         if len(url) > MIN_ABS_FILEURL_LENGTH:
             return url[:-1]
+        else:
+            return url
     scheme_loc, first_path_slash = _find_scheme_and_separator(url)
     if scheme_loc is None:
         # This is a relative path, as it has no scheme
@@ -294,8 +315,14 @@
                 + ['%02X' % o for o in _no_decode_ords])
 _hex_display_map = dict(([('%02x' % o, chr(o)) for o in range(256)]
                     + [('%02X' % o, chr(o)) for o in range(256)]))
+#These entries get mapped to themselves
 _hex_display_map.update((hex,'%'+hex) for hex in _no_decode_hex)
-#These entries get mapped to themselves
+
+# These characters should not be escaped
+_url_safe_characters = set('abcdefghijklmnopqrstuvwxyz'
+                        'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+                        '0123456789' '_.-/'
+                        ';?:@&=+$,%#')
 
 
 def unescape_for_display(url):

